Learn R Programming

OUwie (version 3.0.3)

hOUwie.recon: Reconstruct the marginal probability of discrete node states under the hOUwie model.

Description

Uses the output object from hOUwie to conduct an ancestral state reconstruction of the discrete regime states.

Usage

hOUwie.recon(houwie_obj, 
             nodes      = "all")

Value

recon_matrix

a data.frame containing the marginal probabilities of each state (given by the column).

Arguments

houwie_obj

A list of class "houwie".

nodes

One of "internal", "external", or "all" to represent reconstructing internal nodes, external (tips) nodes, or all nodes. "External" is only useful if the model has hidden states or there is uncertainty in the tip states.

Author

James D. Boyko

Details

Reconstructs the marginal probability of a particular state at a given node. To do this, all possible states at the given node are fixed and the joint likelihoods are evaluated for each possibility. Those joint probabilities are then normalized by the total likelihood so that the probability sums to one.

References

Yang Z. 2006. Computational molecular evolution. Oxford University Press Oxford.