# ## Finally, it is important to get estimates of your parameter uncertainty.
# ## For that we have imported the functions of Brian O'Meara's dentist package
# data(tworegime)
#
#
# ##Third step is to fit our model (here we are using fixed params from a previous ML search):
# p <- c(0.01664314, 0.39631218, 0.18684476, 2.25121568, 0.82495093) # MLE
# pp_cd <- hOUwie(tree, trait, rate.cat = 1, discrete_model = "ER",
# continuous_model = "OUM", nSim = 25, p = p)
#
# ## We can compare this to the result from OUwie on a fixed map to see a difference
# dent_res <- hOUwie.walk(pp_cd, nsteps = 25, debug =TRUE)
# plot(dent_res)
Run the code above in your browser using DataLab