Learn R Programming

OUwie (version 3.0.3)

plot.dentist: Plot the dented samples This will show the univariate plots of the parameter values versus the likelihood as well as bivariate plots of pairs of parameters to look for ridges.

Description

Plot the dented samples This will show the univariate plots of the parameter values versus the likelihood as well as bivariate plots of pairs of parameters to look for ridges.

Usage

# S3 method for dentist
plot(x, ...)

Arguments

x

An object of class dentist

...

Other arguments to pass to plot