# \donttest{
if (FALSE) {
# for the following example we will imagine we have a single discrete character with 3 states.
# these matrices could be assigned to a variable and input into the continuous_model arguement
# in hOUwie for later model comparison.
# there are several model structures but they can be broken into:
# Character dependent (CD) which
getOUParamStructure("OUMA", 3, 1, FALSE)
# Character independent (CID) which can have rate heterogeneity
getOUParamStructure("OUMA", 3, 2, TRUE)
# Hybrid model (HYB) which has both character dependent and character independent rate heterogeneity
getOUParamStructure("OUMA", 3, 2, FALSE)
# from the different uses of the function above, notice how the parameters are associated
# with particular regimes to create the different classes of model. CD has a separate
# parameter for each state, whereas CID always has the different observed states associated
# with the same parameter value. Obviously, different mixes of these are allowed and the
# hybrid model is the most general form which can then be further constrained.
}
# }
Run the code above in your browser using DataLab