Uses the output object from hOUwie to conduct an ancestral state reconstruction of the discrete regime states.
hOUwie.recon(houwie_obj,
nodes = "all")a data.frame containing the marginal probabilities of each state (given by the column).
A list of class "houwie".
One of "internal", "external", or "all" to represent reconstructing internal nodes, external (tips) nodes, or all nodes. "External" is only useful if the model has hidden states or there is uncertainty in the tip states.
James D. Boyko
Reconstructs the marginal probability of a particular state at a given node. To do this, all possible states at the given node are fixed and the joint likelihoods are evaluated for each possibility. Those joint probabilities are then normalized by the total likelihood so that the probability sums to one.
Yang Z. 2006. Computational molecular evolution. Oxford University Press Oxford.